/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.financial.settlement.service;

import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.financial.settlement.entity.CostCheck;
import com.jeeplus.modules.sg.financial.settlement.entity.ProjectBudget;
import com.jeeplus.modules.sg.financial.settlement.mapper.CostCheckMapper;
import com.jeeplus.modules.sg.financial.settlement.mapper.ProjectBudgetMapper;
import org.apache.ibatis.annotations.Param;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.*;

/**
 * 费用检查表Service
 */
@Service
@Transactional(readOnly = true)
public class CostCheckService extends CrudService<CostCheckMapper, CostCheck> {

	@Autowired
	private CostCheckMapper costCheckMapper;

	public static Map<String, CellStyle> styles;

	@Override
	public CostCheck get(String id) {
		return super.get(id);
	}

	@Override
	public List<CostCheck> findList(CostCheck costCheck) {
		return super.findList(costCheck);
	}


	@Override
	public Page<CostCheck> findPage(Page<CostCheck> page, CostCheck costCheck) {
		return super.findPage(page, costCheck);
	}

	@Override
	@Transactional(readOnly = false)
	public void save(CostCheck costCheck) {
		super.save(costCheck);
	}

	@Transactional(readOnly = false)
	public void updateDesignSup(ProjectBudget projectBudget) {
		costCheckMapper.updateDesignSup(projectBudget);
	}

	@Override
	@Transactional(readOnly = false)
	public void delete(CostCheck costCheck) {
		super.delete(costCheck);
	}

	@Transactional(readOnly = false)
	public String insertList(@Param("list") List<ProjectBudget> list) {
		String str = "";
		List<ProjectBudget> insertList = new ArrayList<>();
		List<ProjectBudget> updateList = new ArrayList<>();
		//查找所有的费用检查信息
		List<CostCheck> allList = costCheckMapper.findAllProjectId();
		if (allList.size() > 0) {
			for (ProjectBudget b : list) {
				for (CostCheck a : allList) {
					if (a.getProjectId().equals(b.getProjectId())) {
						b.preInsert();
						updateList.add(b);
					}
				}
			}
			//获得需要更新的项目信息，其它项目信息则插入
			Iterator<ProjectBudget> iterator = list.iterator();
			while (iterator.hasNext()) {
				ProjectBudget next = iterator.next();
				for (ProjectBudget u :
						updateList) {
					if (next.getProjectId().equals(u.getProjectId())) {
						iterator.remove();
					}
				}
			}
			if (list.size() > 0) {
				for (ProjectBudget a :
						list) {
					insertList.add(a);
				}
			}
			if (insertList.size() > 0) {
				for (ProjectBudget p :
						insertList) {
					p.preInsert();
				}
			}
		} else {
			for (ProjectBudget b : list) {
				b.preInsert();
				insertList.add(b);
			}
		}
		//插入
		if (insertList.size() > 0) {
			costCheckMapper.insertList(insertList);
			str += "费用检查信息插入" + insertList.size() + "条-";
		}
		//更新
		if (updateList.size() > 0) {
			costCheckMapper.updateList(updateList);
			str += "费用检查信息更新" + updateList.size() + "条-";
		}
		return str;
	}

	@Transactional(readOnly = false)
	public String insertConstructionList(@Param("list") List<CostCheck> list) {
		String str = "";
		List<CostCheck> updateList = new ArrayList<>();
		List<CostCheck> allList = costCheckMapper.findAllProjectId();
		for (CostCheck costCheck : allList) {
			for (CostCheck c : list) {
				if (costCheck.getProjectId().equals(c.getProjectId())) {
					c.preInsert();
					updateList.add(c);
				}
			}
		}
		if (updateList.size() > 0) {
			costCheckMapper.updateConstructionList(updateList);
			str += "概预算信息更新" + updateList.size() + "条";
		} else {
			str += "概预算信息更新0条";
		}
		return str;
	}

	@Transactional(readOnly = false)
	public void updateDes(CostCheck costCheck) {
		costCheckMapper.updateDes(costCheck);
	}

	@Transactional(readOnly = false)
	public void updateCon(CostCheck costCheck) {
		costCheckMapper.updateCon(costCheck);
	}

	@Transactional(readOnly = false)
	public void updateSup(CostCheck costCheck) {
		costCheckMapper.updateSup(costCheck);
	}
}