/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.financial.settlement.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.audit.information.utils.FreemarkerUtil;
import com.jeeplus.modules.sg.audit.information.utils.ResponseUtil;
import com.jeeplus.modules.sg.financial.settlement.entity.CostCheck;
import com.jeeplus.modules.sg.financial.settlement.entity.ProjectBudget;
import com.jeeplus.modules.sg.financial.settlement.service.CostCheckService;
import com.jeeplus.modules.sg.financial.settlement.service.ProjectBudgetService;
import com.jeeplus.modules.sg.settActivi.entity.SettActivi;
import com.jeeplus.modules.sg.settActivi.entity.SettList;
import com.jeeplus.modules.sys.utils.UserUtils;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.apache.poi.ss.usermodel.Row;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 概预算基本信息维护
 */
@Controller
@RequestMapping(value = "${adminPath}/finance/budget")
public class ProjectBudgetController extends BaseController {

	@Autowired
	private ProjectBudgetService projectBudgetService;

	@Autowired
	private CostCheckService costCheckService;
	
	@ModelAttribute
	public ProjectBudget get(@RequestParam(required=false) String id) {
		ProjectBudget entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectBudgetService.get(id);
		}
		if (entity == null){
			entity = new ProjectBudget();
		}
		return entity;
	}
	
	/**
	 * 列表页面
	 */
	@RequiresPermissions("finance:budget:list")
	@RequestMapping(value = {"list", ""})
	public String list(ProjectBudget projectBudget, Model model) {
		model.addAttribute("projectBudget", projectBudget);
		return "modules/sg/financial/settlement/financeBudgetList";
	}


	/**
	 * 列表数据
	 */
	@ResponseBody
	@RequiresPermissions("finance:budget:list")
	@RequestMapping(value = "data")
	public Map<String, Object> data(ProjectBudget projectBudget, HttpServletRequest request, HttpServletResponse response, Model model,HttpSession session) {

		String st = "";
		String projectId = projectBudget.getProjectId();
		if(StringUtils.isNotBlank(projectId)){
			st = projectId;
		}
//		对查询的id进行处理，若只有一个id设置projectId模糊匹配，若有多个设置projectIds精准匹配
		if(StringUtils.isNotBlank(st)){
			String[] sts = st.split("\\s+");
			if(sts.length == 1){
				projectBudget.setProjectId(sts[0]);
            }else {
				projectBudget.setProjectIds(sts);
				projectBudget.setProjectId(null);
            }
		}
		Page<ProjectBudget> page = projectBudgetService.findPage(new Page<ProjectBudget>(request, response), projectBudget);
		return getBootstrapData(page);
	}


	/**
	 * 财务检查列表页面
	 */
	@RequiresPermissions("finance:budget:list")
	@RequestMapping(value = "financeList")
	public String financeList(ProjectBudget projectBudget, Model model) {
		model.addAttribute("projectBudget", projectBudget);
		return "modules/sg/financial/settlement/financeList";
	}

	/**
	 * 财务检查列表数据
	 */
	@ResponseBody
	@RequiresPermissions("finance:budget:list")
	@RequestMapping(value = "financeData")
	public Map<String, Object> FinanceData(ProjectBudget projectBudget, HttpServletRequest request, HttpServletResponse response, Model model,HttpSession session) {
		String st = "";
		String projectId = projectBudget.getProjectId();
		if(StringUtils.isNotBlank(projectId)){
			st = projectId;
		}
//		对查询的id进行处理，若只有一个id设置projectId模糊匹配，若有多个设置projectIds精准匹配
		if(StringUtils.isNotBlank(st)){
			String[] sts = st.split("\\s+");
			if(sts.length == 1){
				projectBudget.setProjectId(sts[0]);
			}else {
				projectBudget.setProjectIds(sts);
				projectBudget.setProjectId(null);
			}
		}
		Page<ProjectBudget> page = projectBudgetService.findFinancePage(new Page<ProjectBudget>(request, response), projectBudget);
		return getBootstrapData(page);
	}

	/**
	 * 查看，增加，编辑表单页面
	 */
	@RequiresPermissions(value={"finance:budget:view","finance:budget:add","finance:budget:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ProjectBudget projectBudget, Model model) {
		model.addAttribute("projectBudget", projectBudget);
		return "modules/sg/financial/settlement/financeBudgetForm";
	}

	/**
	 * 保存
	 */
	@ResponseBody
	@RequiresPermissions(value={"finance:budget:add","finance:budget:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public AjaxJson save(ProjectBudget projectBudget, Model model) throws Exception{
		AjaxJson j = new AjaxJson();
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(projectBudget);
		if (StringUtils.isNotBlank(errMsg)){
			j.setSuccess(false);
			j.setMsg(errMsg);
			return j;
		}

		double designCost = getStaticDouble(projectBudget.getDesignCost());
		double supervisorCost = getStaticDouble(projectBudget.getSupervisorCost());
		double designContractDiscount = getStaticDouble(projectBudget.getDesignContractDiscount());
		double designTypicalDiscount = getStaticDouble(projectBudget.getDesignTypicalDiscount());
		double supervisorContractDiscount = getStaticDouble(projectBudget.getSupervisorContractDiscount());
		//使用bigDecimal初始化数值，计算防止精度丢失问题
		BigDecimal constant = new BigDecimal("1.06");
		//设计费
		BigDecimal designCostDecimal = new BigDecimal(designCost);
		//监理费
		BigDecimal supervisorCostDecimal = new BigDecimal(supervisorCost);
		//设计合同折扣
		BigDecimal desConDecimal = new BigDecimal(designContractDiscount);
		//设计典设折扣
		BigDecimal desTypeDecimal = new BigDecimal(designTypicalDiscount);
		//监理费合同折扣
		BigDecimal supervisorDecimal = new BigDecimal(supervisorContractDiscount);
		//计算项目设计费应付金额（设计费*设计合同折扣*设计典设折扣*1.06）两位小数
		BigDecimal firResult = designCostDecimal.multiply(desConDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal secResult = firResult.multiply(desTypeDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal designFeePayable = secResult.multiply(constant).setScale(2, BigDecimal.ROUND_HALF_UP);
		projectBudget.setDesignFeePayable(designFeePayable.toString());
		//计算项目监理费应付金额（监理费*监理费合同折扣*1.06）两位小数
		BigDecimal thirdResult = supervisorCostDecimal.multiply(supervisorDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
		BigDecimal supervisionFeePayable = thirdResult.multiply(constant).setScale(2, BigDecimal.ROUND_HALF_UP);
		projectBudget.setSupervisionFeePayable(supervisionFeePayable.toString());
		//新增或编辑表单保存
		projectBudgetService.update(projectBudget);
		costCheckService.updateDesignSup(projectBudget);
		j.setSuccess(true);
		j.setMsg("保存成功");
		return j;
	}
	
	/**
	 * 删除
	 */
	@ResponseBody
	@RequiresPermissions("finance:budget:del")
	@RequestMapping(value = "delete")
	public AjaxJson delete(ProjectBudget projectBudget) {
		AjaxJson j = new AjaxJson();
		projectBudgetService.delete(projectBudget);
		j.setMsg("删除成功");
		return j;
	}


	/**
	 * 删除Model
	 * @param ids
	 * @return
	 */
	@ResponseBody
	@RequiresPermissions("finance:budget:del")
	@RequestMapping(value = "deleteAll")
	public AjaxJson deleteAll(String ids) {
		String idArray[] =ids.split(",");
		ProjectBudget projectBudget = new ProjectBudget();
		CostCheck costCheck = new CostCheck();
		for(String id : idArray){
			projectBudget.setProjectId(id);
			costCheck.setProjectId(id);
			projectBudgetService.delete(projectBudget);
			costCheckService.delete(costCheck);
		}
		AjaxJson j = new AjaxJson();
		j.setMsg("删除成功" );
		return j;
	}

	/**
	 * 导出excel文件
	 */
	@ResponseBody
	@RequestMapping(value = "exportExcel")
	public void exportExcel(ProjectBudget projectBudget, HttpServletRequest request, HttpServletResponse response) {
		Template template = null;
		response.setContentType("text/text;charset=utf-8");
		response.setCharacterEncoding("UTF-8");
		File path = new File(this.getClass().getResource("/").getPath() + "/freemarker");
		Configuration cfg = new Configuration();
		try {
			cfg.setDefaultEncoding("UTF-8");
			//配置cfg对象
			cfg.setDirectoryForTemplateLoading(path);
	//            根据模板名称获取Template对象
			template = cfg.getTemplate("projectBudget.ftl");
		} catch (IOException e) {
			e.printStackTrace();
		}
		Map<String, Object> data = new HashMap<>();
		String st = "";
		String projectId = projectBudget.getProjectId();
		if(StringUtils.isNotBlank(projectId)){
			st = projectId;
		}
//		对查询的id进行处理，若只有一个id设置projectId模糊匹配，若有多个设置projectIds精准匹配
		if(StringUtils.isNotBlank(st)){
			String[] sts = st.split("\\s+");
			if(sts.length == 1){
				projectBudget.setProjectId(sts[0]);
			}else {
				projectBudget.setProjectIds(sts);
				projectBudget.setProjectId(null);
			}
		}
		List<ProjectBudget> list = projectBudgetService.findList(projectBudget);
		data.put("list", list);
		File docFile = new File(path + "\\概预算基本信息管理表.xls");
		FreemarkerUtil.generateFile(data, template, docFile);
		ResponseUtil.docResponse("概预算基本信息管理表.xls", docFile, response);
	}


	/**
	 * 导入Excel数据
	 */
	@ResponseBody
	@RequiresPermissions("finance:budget:import")
    @RequestMapping(value = "import")
   	public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
		AjaxJson j = new AjaxJson();
		try {
			ImportExcel ei = new ImportExcel(file, 2, "概预算基本信息管理");
			List<ProjectBudget> list = getData(ei);
			List<ProjectBudget> costList = new ArrayList<>();
			costList.addAll(list);
			//保存概预算信息
			String s = projectBudgetService.saveList(list);
			//保存费用检查信息
			String b = costCheckService.insertList(costList);
			j.setMsg("导入成功" + s + b);
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg("导入失败！失败信息："+e.getMessage());
		}
		return j;
    }

	/**
	 *excel文件数据读取
	 */
	private List<ProjectBudget> getData(ImportExcel importExcel) throws ParseException {
		int lastRow = importExcel.getLastDataRowNum();
		List<ProjectBudget> list = new ArrayList<>();
		ProjectBudget ProjectBudget;
		String projectType;
		BigDecimal constant = new BigDecimal("1.06");
		for(int i=3; i<lastRow;i++){
			ProjectBudget = new ProjectBudget();
			Row row = importExcel.getRow(i);
			String projectName = (String) importExcel.getCellValue(row,0);
			String projectId = (String) importExcel.getCellValue(row, 1);
			if (StringUtils.isNotBlank(projectName) && StringUtils.isNotBlank(projectId)) {
				ProjectBudget.setProjectName(projectName);
				ProjectBudget.setProjectId(projectId);
				ProjectBudget.setDesignUnit(getString(importExcel, row, 2));
				ProjectBudget.setConstructionUnit(getString(importExcel, row, 3));
				ProjectBudget.setBuildingAddress(getString(importExcel, row, 4));
				ProjectBudget.setBuildingProperty(getString(importExcel, row, 5));
				ProjectBudget.setTotalInvestment(getString(importExcel, row, 6));
				ProjectBudget.setStartTime(getDate(importExcel, row, 7));
				ProjectBudget.setStopTime(getDate(importExcel, row, 8));
				ProjectBudget.setApprovalNumber(getString(importExcel, row, 9));
				ProjectBudget.setConstructionCost(getDouble(importExcel, row, 10));
				ProjectBudget.setInstallationCost(getDouble(importExcel, row, 11));
				ProjectBudget.setEquipmentCost(getDouble(importExcel, row, 12));
				ProjectBudget.setMaterialCost(getDouble(importExcel, row, 13));
				double designCost = getDouble(importExcel, row, 14);
				ProjectBudget.setDesignCost(designCost);
				double supervisorCost = getDouble(importExcel, row, 15);
				ProjectBudget.setSupervisorCost(supervisorCost);
				ProjectBudget.setPreliminaryWorkCost(getDouble(importExcel, row, 16));
				ProjectBudget.setCompensationCost(getDouble(importExcel, row, 17));
				ProjectBudget.setAdministrationCost(getDouble(importExcel, row, 18));
				double designContractDiscount = getDisCount(importExcel, row, 19);
				double designTypicalDiscount = getDisCount(importExcel, row, 20);
				double supervisorContractDiscount = getDisCount(importExcel, row, 21);
				ProjectBudget.setDesignContractDiscount(designContractDiscount);
				ProjectBudget.setDesignTypicalDiscount(designTypicalDiscount);
				ProjectBudget.setSupervisorContractDiscount(supervisorContractDiscount);
				//判断项目是否以b开头，以字母“B”开头，则该项目为成本项目，project_type为0；否则为资本项目，project_type为1
				if (projectId.startsWith("B")) {
					projectType = "0";
				} else {
					projectType = "1";
				}
				ProjectBudget.setProjectType(projectType);
				//使用bigDecimal初始化数值，计算防止精度丢失问题
				//设计费
				BigDecimal designCostDecimal = new BigDecimal(designCost);
				//监理费
				BigDecimal supervisorCostDecimal = new BigDecimal(supervisorCost);
				//设计合同折扣
				BigDecimal desConDecimal = new BigDecimal(designContractDiscount);
				//设计典设折扣
				BigDecimal desTypeDecimal = new BigDecimal(designTypicalDiscount);
				//监理费合同折扣
				BigDecimal supervisorDecimal = new BigDecimal(supervisorContractDiscount);
				//计算项目设计费应付金额（设计费*设计合同折扣*设计典设折扣*1.06）两位小数
				BigDecimal firResult = designCostDecimal.multiply(desConDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
				BigDecimal secResult = firResult.multiply(desTypeDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
				BigDecimal designFeePayable = secResult.multiply(constant).setScale(2, BigDecimal.ROUND_HALF_UP);
				ProjectBudget.setDesignFeePayable(designFeePayable.toString());
				//计算项目监理费应付金额（监理费*监理费合同折扣*1.06）两位小数
				BigDecimal thirdResult = supervisorCostDecimal.multiply(supervisorDecimal).setScale(2, BigDecimal.ROUND_HALF_UP);
				BigDecimal supervisionFeePayable = thirdResult.multiply(constant).setScale(2, BigDecimal.ROUND_HALF_UP);
				ProjectBudget.setSupervisionFeePayable(supervisionFeePayable.toString());

				list.add(ProjectBudget);
			}
		}
		return list;
	}

	/**
	 *获取excel double类型的值
	 */
	private double getDouble(ImportExcel importExcel,Row row,int col){
		double d = 0.00;
        Object o = importExcel.getCellValue(row,col);
        if(!o.equals("") && !o.equals("-")){
        	d = Double.parseDouble(o.toString());
		}
      return d;
	}

	/**
	 *获取excel double类型的值(折扣比率值，不能小于1.0)
	 */
	private double getDisCount(ImportExcel importExcel,Row row,int col){
		double d = 1.00;
		Object o = importExcel.getCellValue(row,col);
		if(!o.equals("")){
			d = Double.parseDouble(o.toString());
		}
		return d;
	}

	/**
	 *获取excel date类型的值
	 */
	private Date getDate(ImportExcel importExcel,Row row,int col) throws ParseException {
		Date d = null;
		Object o = importExcel.getCellValue(row,col);
		if(!o.equals("")){
			SimpleDateFormat simpleDateFormat=new SimpleDateFormat("yyyy-MM-dd");
			d = simpleDateFormat.parse(o.toString());
		}
		return d;
	}

	/**
	 * 获取excel String类型的值
	 * */
	private String getString(ImportExcel importExcel,Row row,int col) throws ParseException {
		String str = "";
		Object o = importExcel.getCellValue(row, col);
		if (null != o && !"".equals(o)) {
			str = String.valueOf(o);
		}
		return str;
	}


	//校验数据表格是否正确
	public static Boolean getMainJudge(ImportExcel importExcel){
		Boolean flag = false;
		Row row = importExcel.getRow(0);
		String mainCode = (String) importExcel.getCellValue(row,0);
		if (!mainCode.contains("项目定义")){
			flag = true;
		}
		String lineItem = (String)importExcel.getCellValue(row,1);
		if (!lineItem.contains("单位")){
			flag = true;
		}
		return  flag;
	}

	/**
	 *获取excel double类型的值(折扣比率值，不能小于1.0)
	 */
	private double getStaticDouble(Double a){
		double d = 1.00;
		if (null != a) {
			return a;
		}
		return d;
	}

}